@@ -40,6 +40,7 @@ static int arv_option_duration_s = -1;
 static char *arv_option_uv_usb_mode = NULL;
 static gboolean arv_option_show_version = FALSE;
 static gboolean arv_option_gv_allow_broadcast_discovery_ack = FALSE;
+static char *arv_option_gv_discovery_interface = NULL;
 
 /* clang-format off */
 static const GOptionEntry arv_option_entries[] =
@@ -211,6 +212,11 @@ static const GOptionEntry arv_option_entries[] =
                 "Allow broadcast discovery ack",
 		NULL
 	},
+	{
+		"gv-discovery-interface",               '\0', 0, G_OPTION_ARG_STRING,
+		&arv_option_gv_discovery_interface,     "Discovery using the interface",
+		"<interface>"
+        },
 	{
 		"debug", 				'd', 0, G_OPTION_ARG_STRING,
 		&arv_option_debug_domains, 		"Debug output selection",
@@ -475,6 +481,9 @@ main (int argc, char **argv)
         if (arv_option_gv_allow_broadcast_discovery_ack)
                 arv_set_interface_flags ("GigEVision", ARV_GV_INTERFACE_FLAGS_ALLOW_BROADCAST_DISCOVERY_ACK);
 
+	if (arv_option_gv_discovery_interface)
+		arv_gv_interface_set_discovery_interface_name (arv_option_gv_discovery_interface);
+
 	arv_enable_interface ("Fake");
 
 	arv_debug_enable (arv_option_debug_domains);
