@@ -1,21 +1,21 @@
 /* Aravis - Digital camera library
  *
- * Copyright © 2009-2022 Emmanuel Pacaud
+ * Copyright © 2009-2025 Emmanuel Pacaud <emmanuel.pacaud@free.fr>
+ *
+ * SPDX-License-Identifier: LGPL-2.1-or-later
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
+ * version 2.1 of the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU Lesser General
- * Public License along with this library; if not, write to the
- * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
- * Boston, MA 02110-1301, USA.
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, see <http://www.gnu.org/licenses/>.
  *
  * Author: Emmanuel Pacaud <emmanuel.pacaud@free.fr>
  */
@@ -387,7 +387,7 @@ _find_frame_data (ArvGvStreamThreadData *thread_data,
                 if (thread_data->callback != NULL)
                         thread_data->callback (thread_data->callback_data,
                                                ARV_STREAM_CALLBACK_TYPE_BUFFER_DONE,
-                                               frame->buffer);
+                                               buffer);
                 return NULL;
         }
 
@@ -1830,8 +1830,8 @@ arv_gv_stream_class_init (ArvGvStreamClass *gv_stream_class)
         /**
          * ArvGvStream:socket-buffer-size:
          *
-         * Size in bytes of the incoming socket buffer. A greater value helps to lower the number of missings packets,
-         * as the expense of an increased memory usage.
+         * Size in bytes of the incoming socket buffer. A greater value helps to lower the number of missing packets,
+         * as the expense of an increased memory usage. If the value is not strictly positive, this setting is ignored.
          */
 	g_object_class_install_property (
 		object_class, ARV_GV_STREAM_PROPERTY_SOCKET_BUFFER_SIZE,
