@@ -1,21 +1,21 @@
 /* Aravis - Digital camera library
  *
- * Copyright © 2009-2022 Emmanuel Pacaud
+ * Copyright © 2009-2025 Emmanuel Pacaud <emmanuel.pacaud@free.fr>
+ *
+ * SPDX-License-Identifier: LGPL-2.1-or-later
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
+ * version 2.1 of the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU Lesser General
- * Public License along with this library; if not, write to the
- * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
- * Boston, MA 02110-1301, USA.
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, see <http://www.gnu.org/licenses/>.
  *
  * Author: Emmanuel Pacaud <emmanuel.pacaud@free.fr>
  */
@@ -804,6 +804,70 @@ ArvGstCapsInfos arv_gst_caps_infos[] = {
 	},
 };
 
+typedef struct {
+	const char *vendor_selection;
+	const char *model_selection;
+} ArvBigEndianPixelFormatInfos;
+
+/* Some cameras seem to incorrectly transmit multi bytes image data in big endian order, even if GevSCPSBigEndian is not
+ * set. In this case, we need to convert the pixel format to a big endian GstCaps. We maintain a list of such cameras
+ * here. */
+
+static const ArvBigEndianPixelFormatInfos arv_big_endian_pixel_format_infos[] = {
+	{ .vendor_selection = "Point Grey Research", .model_selection = "Blackfly BFLY-PGE-31S4M" },
+	{ .vendor_selection = "Point Grey Research", .model_selection = "Blackfly BFLY-PGE-14S2C" }
+};
+
+/* Only Mono16 is handled here, as we did not seen other cases of misbehaving devices with other pixel format for now. */
+
+ArvGstCapsInfos big_endian_gst_caps_infos[] = {
+	{
+		ARV_PIXEL_FORMAT_MONO_16,
+		"video/x-raw, format=(string)GRAY16_BE",
+		"video/x-raw",		"GRAY16_BE",
+		"video/x-raw-gray, bpp=(int)16, depth=(int)16",
+		"video/x-raw-gray",	16,	16,	0
+	},
+};
+
+static gboolean
+_use_big_endian_pixel_format (const char *vendor_name,
+                              const char *model_name)
+{
+        guint i;
+
+	for (i = 0; i < G_N_ELEMENTS (arv_big_endian_pixel_format_infos); i++) {
+		if ((g_pattern_match_simple (arv_big_endian_pixel_format_infos[i].vendor_selection, vendor_name) == TRUE) &&
+		    (g_pattern_match_simple (arv_big_endian_pixel_format_infos[i].model_selection, model_name) == TRUE)) {
+			return TRUE;
+		}
+	}
+
+	return FALSE;
+}
+
+static const char *
+pixel_format_to_gst_caps_string (ArvPixelFormat pixel_format,
+				 const ArvGstCapsInfos *infos,
+				 guint n_elements)
+{
+	while (n_elements > 0) {
+		if (infos->pixel_format == pixel_format)
+			break;
+		infos ++;
+		n_elements --;
+	}
+
+	if (n_elements == 0) {
+		return NULL;
+	}
+
+	arv_debug_misc ("[PixelFormat::to_gst_caps_string] 0x%08x -> %s",
+		      pixel_format, infos->gst_caps_string);
+
+	return infos->gst_caps_string;
+}
+
 /**
  * arv_pixel_format_to_gst_caps_string:
  * @pixel_format: a pixel format
@@ -813,21 +877,53 @@ ArvGstCapsInfos arv_gst_caps_infos[] = {
 const char *
 arv_pixel_format_to_gst_caps_string (ArvPixelFormat pixel_format)
 {
-	int i;
+	const char *caps_string;
 
-	for (i = 0; i < G_N_ELEMENTS (arv_gst_caps_infos); i++)
-		if (arv_gst_caps_infos[i].pixel_format == pixel_format)
-			break;
+	caps_string = pixel_format_to_gst_caps_string(pixel_format,
+						      arv_gst_caps_infos,
+						      G_N_ELEMENTS (arv_gst_caps_infos));
 
-	if (i == G_N_ELEMENTS (arv_gst_caps_infos)) {
+	if (caps_string == NULL) {
 		arv_warning_misc ("[PixelFormat::to_gst_caps_string] 0x%08x not found", pixel_format);
-		return NULL;
 	}
 
-	arv_debug_misc ("[PixelFormat::to_gst_caps_string] 0x%08x -> %s",
-		      pixel_format, arv_gst_caps_infos[i].gst_caps_string);
+	return caps_string;
+}
 
-	return arv_gst_caps_infos[i].gst_caps_string;
+#include <stdio.h>
+
+/**
+ * arv_pixel_format_to_gst_caps_string_full:
+ * @pixel_format: a pixel format
+ * @vendor_name: vendor name
+ * @model_name: model name
+ * Return value: (nullable): a gstreamer caps string describing the given @pixel_format, NULL if not found.
+ *
+ * The returned caps string defines a little endian pixel format, unless an exception has been detected. In this case, a
+ * big endian caps is returned.
+ *
+ * Since: 0.8.32
+ */
+
+const char *
+arv_pixel_format_to_gst_caps_string_full (ArvPixelFormat pixel_format,
+                                          const char *vendor_name,
+                                          const char *model_name)
+{
+        const char *caps_string;
+
+	caps_string = NULL;
+
+	if (_use_big_endian_pixel_format (vendor_name, model_name))
+                caps_string = pixel_format_to_gst_caps_string (pixel_format,
+                                                               big_endian_gst_caps_infos,
+                                                               G_N_ELEMENTS (big_endian_gst_caps_infos));
+
+	if (caps_string != NULL) {
+		return caps_string;
+	}
+
+	return arv_pixel_format_to_gst_caps_string (pixel_format);
 }
 
 ArvPixelFormat
