@@ -1,21 +1,21 @@
 /* Aravis - Digital camera library
  *
- * Copyright © 2009-2022 Emmanuel Pacaud
+ * Copyright © 2009-2025 Emmanuel Pacaud <emmanuel.pacaud@free.fr>
+ *
+ * SPDX-License-Identifier: LGPL-2.1-or-later
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
+ * version 2.1 of the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU Lesser General
- * Public License along with this library; if not, write to the
- * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
- * Boston, MA 02110-1301, USA.
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, see <http://www.gnu.org/licenses/>.
  *
  * Authors: Emmanuel Pacaud <emmanuel.pacaud@free.fr>
  *          Václav Šmilauer <eu@doxos.eu>
@@ -24,6 +24,7 @@
 #include <arvnetworkprivate.h>
 #include <arvdebugprivate.h>
 #include <arvmiscprivate.h>
+#include <errno.h>
 
 #ifndef G_OS_WIN32
 	#include <ifaddrs.h>
@@ -446,23 +447,60 @@ arv_network_interface_free(ArvNetworkInterface *a)
 	g_free (a);
 }
 
-
 gboolean
 arv_socket_set_recv_buffer_size (int socket_fd, gint buffer_size)
 {
-	int result;
-
-#ifndef G_OS_WIN32
-	result = setsockopt (socket_fd, SOL_SOCKET, SO_RCVBUF, &buffer_size, sizeof (buffer_size));
+#ifdef G_OS_WIN32
+	DWORD _buffer_size;
+	DWORD buffer_size_reported;
 #else
-	{
-		DWORD _buffer_size=buffer_size;
-		result = setsockopt (socket_fd, SOL_SOCKET, SO_RCVBUF,
-				     (const char*) &_buffer_size, sizeof (_buffer_size));
+	gint _buffer_size;
+	gint buffer_size_reported;
+#endif
+	int result;
+	socklen_t optlen;
+
+        _buffer_size = buffer_size;
+        optlen = sizeof(buffer_size_reported);
+
+	result = setsockopt (socket_fd, SOL_SOCKET, SO_RCVBUF,
+			     (const char *) &_buffer_size, sizeof (_buffer_size));
+	if(result != 0) {
+	    arv_warning_interface ("[set_recv_buffer_size] Setting socket buffer to %d bytes failed (%s)",
+                                  _buffer_size, strerror(errno));
+	    return FALSE;
 	}
+
+        /* setsockopt() succeeded, but sometimes the requested size is not actually be set. Ask
+         * to see the new setting to confirm. */
+        result = getsockopt (socket_fd, SOL_SOCKET, SO_RCVBUF, (char *) &buffer_size_reported, &optlen);
+        if (result != 0) {
+                arv_warning_interface ("[set_recv_buffer_size] Read of socket buffer size (SO_RCVBUF) failed (%s)",
+                                       strerror(errno));
+                return FALSE;
+        }
+        g_assert (optlen == sizeof (buffer_size_reported));
+
+	if(buffer_size_reported < buffer_size)
+        {
+#ifndef G_OS_WIN32
+                arv_warning_interface ("[set_recv_buffer_size] Unexpected socket buffer size (SO_RCVBUF):"
+                                       " actual %d < expected %d bytes"
+                                       "\nYou might see missing packets and timeouts"
+                                       "\nMost likely /proc/sys/net/core/rmem_max is too low"
+                                       "\nSee the socket(7) manpage\n",
+                                       buffer_size_reported, buffer_size);
+#else
+                arv_warning_interface ("[set_recv_buffer_size] Unexpected socket buffer size (SO_RCVBUF):"
+                                       " actual %d < expected %d bytes"
+                                       "\nYou might see missing packets and timeouts",
+                                       buffer_size_reported, buffer_size);
 #endif
 
-	return result == 0;
+                return FALSE;
+        }
+
+	return TRUE;
 }
 
 
